unit COMM;

/////////////////////////////////////////////////////////////////////
///////  Prcdures de communication pour le port COM    ///////////////
//////////////////////////////////////////////////////////////////////

interface

uses Classes,SyncObjs;

// D'aprs les procdures de Types.pas et SeekTelescope.pas de PRiSM

var
   hCom:Integer;     // Dfinira le moyen d'atteindre le port
   NumCOM:Integer;   // numro du port srie COM utilis pour joindre la monture
   MAJ:Boolean;      // Condition de mise  jour du programme via le boot
   SectionCritique:TCriticalSection;
Const
   // ces constantes vont servir comme slection du port srie.
   AccesCom1:boolean=False;
   AccesCom2:boolean=False;
   AccesCom3:boolean=False;
   AccesCom4:boolean=False;
   AccesCom5:boolean=False;
   AccesCom6:boolean=False;
   AccesCom7:boolean=False;
   AccesCom8:boolean=False;
   AccesCom9:boolean=False;
   AccesCom10:boolean=False;
   AccesCom11:boolean=False;
   AccesCom12:boolean=False;

   // Constante pour communication avec les axes
   AxeZero:byte=$E0;
procedure DestroySectionCritique;
procedure EnumComPorts(Ports: TStrings);
function AccesComAllowance(Com:Integer):Boolean;
procedure AccesCom(Com:Integer);
Procedure Open_Cod_Eng(Var resultat:Boolean);
function  deviceready(device: byte) : integer;
function  ReadEncoder(octet,device: byte) : integer;
function StopSlewing(device: byte) : boolean;
function sendcommand(command: ShortString; number: byte; device: byte) : ShortString;

implementation

uses
   SysUtils,Dialogs,
   Windows,forms,Registry;

procedure InitSectionCritique;
begin
   SectionCritique.Create;
end;

procedure DestroySectionCritique;
begin
   SectionCritique.Release;
end;

// Enumration des pors srie dispo sur l'ordi
procedure EnumComPorts(Ports: TStrings);
var
  KeyHandle: HKEY;
  ErrCode, Index: Integer;
  ValueName, Data: string;
  ValueLen, DataLen, ValueType: DWORD;
  TmpPorts: TStringList;
begin
  ErrCode := RegOpenKeyEx(
    HKEY_LOCAL_MACHINE,
    'HARDWARE\DEVICEMAP\SERIALCOMM',
    0,
    KEY_READ,
    KeyHandle);

  TmpPorts := TStringList.Create;
  try
    Index := 0;
    repeat
      ValueLen := 256;
      DataLen := 256;
      SetLength(ValueName, ValueLen);
      SetLength(Data, DataLen);
      ErrCode := RegEnumValue(
        KeyHandle,
        Index,
        PChar(ValueName),
{$IFDEF DELPHI_4_OR_HIGHER}
        Cardinal(ValueLen),
{$ELSE}
        ValueLen,
{$ENDIF}
        nil,
        @ValueType,
        PByte(PChar(Data)),
        @DataLen);

      if ErrCode = ERROR_SUCCESS then
      begin
        SetLength(Data, DataLen);
        Delete(Data,1,3);
        TmpPorts.Add(Data);
        Inc(Index);
      end;

    until (ErrCode <> ERROR_SUCCESS) ;

    TmpPorts.Sort;
    Ports.Assign(TmpPorts);
  finally
    RegCloseKey(KeyHandle);
    TmpPorts.Free;
  end;

end;

// En gros, vrifie si un port srie a t dfini ( entre 1 et 10 ) :
function AccesComAllowance(Com:Integer):Boolean;
begin
 result:=((Com=1) and (not AccesCom3) and (not AccesCom1)) or
         ((Com=3) and (not AccesCom1) and (not AccesCom3)) or
         ((Com=2) and (not AccesCom4) and (not AccesCom2)) or
         ((Com=4) and (not AccesCom2) and (not AccesCom4)) or
         ((Com=5) and (not AccesCom5)) or
         ((Com=6) and (not AccesCom6)) or
         ((Com=7) and (not AccesCom7)) or
         ((Com=8) and (not AccesCom8)) or
         ((Com=9) and (not AccesCom9)) or
         ((Com=10) and (not AccesCom10)) or
         ((Com=11) and (not AccesCom11)) or
         ((Com=12) and (not AccesCom12));
end;

// Permet de dfinir le port qui a t slectionn
procedure AccesCom(Com:Integer);
begin
 case Com of
  1:AccesCom1:=Not AccesCom1;
  2:AccesCom2:=Not AccesCom2;
  3:AccesCom3:=Not AccesCom3;
  4:AccesCom4:=Not AccesCom4;
  5:AccesCom5:=Not AccesCom5;
  6:AccesCom6:=Not AccesCom6;
  7:AccesCom7:=Not AccesCom7;
  8:AccesCom8:=Not AccesCom8;
  9:AccesCom9:=Not AccesCom9;
  10:AccesCom10:=Not AccesCom10;
  11:AccesCom11:=Not AccesCom11;
  12:AccesCom12:=Not AccesCom12;
 end;
end;

// Procdure pour ouvrir le port COM ; cette procdure permet de changer
// la variable *rsultat* envoye en paramtre pour stipuler si l'ouverture
// du port a russi ou chou ...
Procedure Open_Cod_Eng(Var resultat:Boolean);
Var
  ComCh:String;
  DCB:TDCB;
  CommTimeouts:TCommTimeouts;
begin
  ComCh:='COM'+IntToStr(NumCOM);  // Definir le port COM comme COM1, COM2, ...

  // Cration du HANDLE  partir duquel le port COM dsir pourra tre joint :
  hCom:=CreateFile(PChar(ComCh),GENERIC_READ or GENERIC_WRITE,0,Nil,OPEN_EXISTING,0,0);
  resultat:=GetCommState(hCom,DCB);   // Regarde si la fonction a chou ou non en tant que dfinition du port srie

  if resultat then  // Si pas d'chec, ouverture du port srie ( spcifique  chaque langage ... )
   begin
   DCB.baudRate:=19200;
   DCB.ByteSize:=8    ;
   DCB.Parity  :=0    ;
   DCB.StopBits:=0    ;

   SetCommState(hCom,DCB);       //return IO
   SetCommMask (hCom,EV_RXCHAR); //return IO

   With CommTimeouts do
     begin
     ReadIntervalTimeout        :=0;
     ReadTotalTimeoutMultiplier :=0;
     ReadTotalTimeoutConstant   :=500;
     WriteTotalTimeoutMultiplier:=0;
     WriteTotalTimeoutConstant  :=250;
     end;
   SetCommTimeouts(hCom,CommTimeouts);
   end;
end;

procedure FlushWriteRead(SendBuff:array of byte;NumbyteWrite:byte;
                           Var NbOfBytesWrite:longword;
                           Var RecBuff:array of byte;NumbyteRead:byte;
                           var NbOfBytesRead:longword);
begin
   SectionCritique.Enter;
   FlushFileBuffers(hCom);
   WriteFile(hCom,SendBuff,NumbyteWrite,NbOfBytesWrite,nil);
   ReadFile(hCom,RecBuff,NumbyteRead,NbOfBytesRead,nil);
   SectionCritique.Leave;
end;

function deviceready(device: byte) : integer;

var
    RecBuff : array [0..7] of byte;
    SendBuff : array [0..3] of byte;
    abyte,numbyte,retry: byte;
    NbOfBytesWrite,NbOfBytesRead:LongWord;

begin
// FlushFileBuffers(hCom);
 numbyte := 2;
 SendBuff[0] := AxeZero + device;
 SendBuff[1] := 255;
 retry := 3;
 repeat
   FlushWriteRead(SendBuff,numbyte,NbOfBytesWrite,RecBuff[0],1,NbOfBytesRead);
//        WriteFile(hCom,SendBuff,numbyte,NbOfBytesWrite,nil);
//        ReadFile(hCom,RecBuff[0],1,NbOfBytesRead,nil);
        abyte := RecBuff[0];
        retry := retry -1;
//        Application.ProcessMessages;
 until ( retry=0 ) or (NbOfBytesRead <> 0);
 if abyte = 1 then deviceready := 1
  else deviceready := 0;
 if(NbOfBytesRead = 0) then
    deviceready := -1;
end;





function ReadEncoder(octet,device: byte) : integer;

var
    RecBuff : array [0..7] of byte;
    SendBuff : array [0..3] of byte;
    abyte,numbyte,retry: byte;
    NbOfBytesWrite,NbOfBytesRead:LongWord;

begin
   retry := 10;
   repeat
//        FlushFileBuffers(hCom);
   numbyte := 2;
   SendBuff[0] := AxeZero + device;
   SendBuff[1] := $F1 + octet;
//        WriteFile(hCom,SendBuff,numbyte,NbOfBytesWrite,nil);
//        ReadFile(hCom,RecBuff[0],1,NbOfBytesRead,nil);
   FlushWriteRead(SendBuff,numbyte,NbOfBytesWrite,RecBuff[0],1,NbOfBytesRead);
   abyte := RecBuff[0];
   retry := retry -1;
 until ( retry=0 ) or (NbOfBytesRead <> 0);
 if(NbOfBytesRead = 0) then
    ReadEncoder := -1
 else
    ReadEncoder := abyte;
end;


function StopSlewing(device: byte) : boolean;

var
    RecBuff : array [0..7] of byte;
    SendBuff : array [0..3] of byte;
    numbyte: byte;
    NbOfBytesWrite,NbOfBytesRead:LongWord;

begin
 StopSlewing := false;
// FlushFileBuffers(hCom);
 numbyte := 2;
 SendBuff[0] := AxeZero + device;
 SendBuff[1] := 240;
// WriteFile(hCom,SendBuff,numbyte,NbOfBytesWrite,nil);
// ReadFile(hCom,RecBuff[0],1,NbOfBytesRead,nil);
   FlushWriteRead(SendBuff,numbyte,NbOfBytesWrite,RecBuff[0],1,NbOfBytesRead);
 if(NbOfBytesRead = 0) then
    ShowMessage('Erreur de communication du device '+inttostr(device)+' '+inttostr(NumCOM)+'!')
 else StopSlewing := true;
end;

function sendcommand(command: ShortString; number : byte; device: byte) : ShortString;

var
    NbOfBytesWrite,NbOfBytesRead:LongWord;
    RecBuff : array [0..79] of byte;
    SendBuff : array [0..6] of byte;
    abyte,i,c,checksum: byte;
    astring: ShortString;
    achar: char;

begin
 sendcommand := '';
 astring := '';
 astring := chr(AxeZero + device);
 astring := astring + command;
 checksum := 0;
 for i:= 1 to 6 do
  begin
   SendBuff[i-1] := ord(astring[i]);
   checksum := checksum + ord(astring[i]);
  end;
 SendBuff[6] := checksum;
// FlushFileBuffers(hCom);
// WriteFile(hCom,SendBuff,7,NbOfBytesWrite,nil);
// ReadFile(hCom, RecBuff, number + 1, NbOfBytesRead, nil);   //receive ACK
   FlushWriteRead(SendBuff,7,NbOfBytesWrite,RecBuff[0],number+1,NbOfBytesRead);
 abyte := RecBuff[0];

 sendcommand := inttostr(abyte);

 astring := '';
 if (number > 0) and (abyte = 6) then
  begin
   c:=1;
   while c<>number+1 do
    begin
     achar := chr(RecBuff[c]);
     if (number = 80) and (achar = chr(13)) then
      c:=80
     else
      astring := astring + achar;
     c:=c+1;
    end;
    sendcommand := astring;
  end;
end;

initialization
begin
   SectionCritique:=TCriticalSection.Create;
end;


end.
